'use strict';


angular.module('dnuiPlugin')
    .controller('ImagesCtrl', ['$scope', '$rootScope', 'ImagesResource', 'BackupResource',
        function ($scope, $rootScope, ImagesResource, BackupResource) {

            //this is use for refresh the image list
            var refreshImages = true;

            /*
             0 => unknown
             1 => fetching..
             2 => ok
             */
            $scope.callServerStatus = 0;

            var getImages = function (numberPage) {
                $scope.callServerStatus = 1;
                //refreshing so change to false
                refreshImages = false;

                //desactivate the delete all button, only will be visible if all image where verified
                $scope.deleteAllButton = 0;
                $scope.images = [];
                if (_.isUndefined(numberPage)) {
                    numberPage = {};
                } else {
                    numberPage = {numberPage: numberPage};
                }

                var countButtonDeleteAllButton = 0; //begin of the count

                ImagesResource.readByOptions(numberPage).$promise.then(function (images) {

                    $scope.images = images;
                    $scope.callServerStatus = 2;
                    countButtonDeleteAllButton = images.length; //all original images
                    $scope.deleteAllButton = 2; //waiting all image status
                    angular.forEach($scope.images, function (image, key) {

                        //checking status
                        image.status.used = -1;
                        image.status.inServer = -1;
                        angular.forEach(image.imageSizes, function (imageSize) {
                            imageSize.status.used = -1;
                            imageSize.status.inServer = -1;
                            $scope.deleteAllButton++; //all sizes
                        });

                        ImagesResource.verifyUsedById({id: image['id']}).$promise.then(function (statusSizes) {

                            _.each(statusSizes[image['id']], function (status, sizeName) {

                                if ($scope.options.galleryCheck) {
                                    if (!_.isUndefined($scope.galleriesSizes[image['id']])) {

                                        if ($scope.galleriesSizes[image['id']]['sizes'].indexOf(sizeName) > -1) {
                                            status.used = 1;
                                        }
                                    }
                                }

                                if (sizeName === 'original') {
                                    image['status'] = status;
                                } else {
                                    _.findWhere(image.imageSizes, {sizeName: sizeName})['status'] = status;
                                }
                                countButtonDeleteAllButton--;
                                if (countButtonDeleteAllButton == 0) {
                                    $scope.deleteAllButton = 1; //delete all button
                                }

                            });

                        });

                    });
                });
            };


            $rootScope.$on('options', function (event, options) {

                $scope.options = options;

                refreshImages = true;


            });

            $rootScope.$on('tabImages', function () {

                if (refreshImages) {
                    ImagesResource.count().$promise.then(function (count) {
                        $scope.totalImages = count['0'];
                    });

                    if ($scope.options.galleryCheck) {

                        if (_.isUndefined($scope.galleriesSizes)) {
                            $scope.callServerStatus = 1;
                            ImagesResource.galleries().$promise.then(function (galleriesSizes) {

                                $scope.galleriesSizes = galleriesSizes;
                                getImages();
                            });
                        } else {
                            getImages();
                        }

                    } else {
                        getImages();
                    }

                }
            });

            $rootScope.$on('pageChange', function (event, numberPage) {

                getImages(numberPage);

            });

            $rootScope.$on('backup', function () {

                refreshImages = true;

            });

            $scope.delete = function (id, sizeName, image) {

                var deleteFunction = function (id, sizeName, image) {
                    image.status.used = 3;
                    angular.forEach(image.imageSizes, function (imageSize, sizeName) {
                        imageSize.status.used = 3;
                    });

                    ImagesResource.deleteByIdAndSize({id: id, sizeName: sizeName}).$promise.then(function (status) {
                        image.status = status;
                        if (sizeName == 'original') {
                            angular.forEach(image.imageSizes, function (imageSize, sizeName) {
                                imageSize.status = status;
                            });
                        }
                    });
                };

                //only if image is not used
                if (image.status.used == 0 && $scope.unusedImageSizesForOriginal(image)) {


                    if ($scope.options.backup) {
                        image.status.used = 4;
                        BackupResource.make({id: id, sizeName: sizeName}).$promise.then(function (status) {
                            if (status.inServer > 0) {
                                deleteFunction(id, sizeName, image);
                            } else {
                                image.status = status;
                            }
                        });
                    } else {
                        deleteFunction(id, sizeName, image);
                    }

                    refreshImages=true;

                }
            };

            $scope.unusedImageSizesForOriginal = function (image) {
                //special case of original

                if (image.sizeName === 'original') {

                    for (var imageSize in image.imageSizes) {
                        if (image.imageSizes[imageSize].status.used == 1) {
                            return false;
                        }
                    }

                }

                return true;
            }

            $scope.deleteAll = function () {

                $scope.deleteAllButton = false;

                angular.forEach($scope.images, function (image, key) {

                    if (image.sizeName === 'original' && $scope.options.ignoreSizes.length === 0 && $scope.unusedImageSizesForOriginal(image)) {

                        $scope.delete(image.id, image.sizeName, image);

                    } else {

                        angular.forEach(image.imageSizes, function (imageSize, sizeName) {

                            if ($scope.options.ignoreSizes.indexOf(imageSize.sizeName) == -1 && imageSize.status.used == 0) {
                                $scope.delete(image.id, imageSize.sizeName, imageSize);
                            }

                        });

                    }
                });
            };


        }
    ]);
